
package zoomagazin;
import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;


public class Catalogue {

    private LinkedList<Product> products;
    private ObjectInputStream input;
    private ObjectOutputStream output;


    public Catalogue(){
        renewList();
    }

    public Catalogue( Catalogue cat ) throws ZooException{
        throw new ZooException("Copying a Catalogue is a not allowed operation!");
    }

    public Product getProduct( int index ) throws ZooException{
        try{
            return products.get( index );
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while getting product: Index " + index
                    + " is out of bounds. There are " + size() + " available products" );
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while getting product at index " + index );
        }
    }

    public boolean addProduct( Product product ){
        if( product == null ) return false;

        return products.add( product );
    }

    public void insertProduct( int dest, Product product ) throws ZooException{
        try{
            products.add( dest, product );
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while inserting product: Index " + dest
                    + " is out of bounds. There are " + size() + " available products" );
        }
    }

	public boolean addProduct( int quantity, double price, double vipPrice, ProductSpec spec ) throws ZooException{
		int id = -1;
		Iterator iter = null;
		while( true ){
			id += 1;
			iter = products.iterator();
			while( iter.hasNext() ){
				if( ((Product) iter.next()).getId() == id ) break;
			}
			if( ! iter.hasNext() ) break;
		}
		
		Product product = new Product( id + 1, quantity, price, vipPrice, spec );
		return products.add( product );
	}

	public Vector<String> getListProductNames(){
        Vector<String> lstProductNames = new Vector<String>();

		Iterator iter = products.iterator();
		while( iter.hasNext() ){
			lstProductNames.add( ((Product) iter.next()).getProductName() );
		}

		return lstProductNames;
	}

    public boolean removeProduct( int index ) throws ZooException{
        try{
            return products.remove( index ) != null;
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while removing product: Index " + index
                    + " is out of bounds. There are " + size() + " available products" );
        }
    }

    public void openDataBase( String file ) throws ZooException{
        renewList();
        addDataBase( file );
    }

    public void addDataBase( String file ) throws ZooException{
        openFileForInput( file );

        try{
            while( true ){
                products.add( (Product) input.readObject() );
            }
        }
        catch( EOFException ex ){
            // File finished.
            return;
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while reading from file " + file );
        }
        finally{
            closeFileForInput();
        }
    }

    public void saveDataBase( String file ) throws ZooException{
        try{
            openFileForOutput(file);

            Iterator iter = products.iterator();
            while( iter.hasNext() ){
                output.writeObject( iter.next() );
            }
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while saving repository to file " + file );
        }
        finally{
            closeFileForOutput();
        }
    }

    public String toString(){
        if( products.isEmpty() ) return new String( "" );

        StringBuffer buffer = new StringBuffer();

        Iterator iter = products.iterator();
        int number = 0;
        while( iter.hasNext() ){
            buffer.append( "\nProduct No " + ++number + ":\n" );
            buffer.append( iter.next() );
        }

        return buffer.toString();
    }

    public int size(){
        return products.size();
    }


    //// Private methods
    //
    private void openFileForInput( String file ) throws ZooException{
        try{
            input = new ObjectInputStream( new FileInputStream( file ) );
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while opening file " + file + " for input." );
        }
    }

    private void openFileForOutput( String file ) throws ZooException{
        try{
            output = new ObjectOutputStream( new FileOutputStream( file ) );
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while opening file " + file + " for output" ) ;
        }
    }

    private void closeFileForInput() throws ZooException{
        try {
            if( input != null ){
                input.close();
            }
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while closing file for input." );
        }
    }

    private void closeFileForOutput() throws ZooException{
        try {
            if( output != null ){
                output.flush();
                output.close();
            }
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while closing file for output." );
        }
    }

    private void renewList(){
        products = new LinkedList<Product>();
    }

}
