
package zoomagazin;

import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Iterator;
import java.util.LinkedList;


public class UserManager {
    private LinkedList<User> users;
    private ObjectInputStream input;
    private ObjectOutputStream output;


    public UserManager(){
        renewList();
    }

    public UserManager( UserManager um ) throws ZooException{
        users = (LinkedList<User>) um.getUsers().clone();
    }

    public LinkedList<User> getUsers(){
        return users;
    }

    public User getUser( int index ) throws ZooException{
        try{
            return users.get( index );
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while getting user: Index " + index
                    + " is out of bounds. There are " + size() + " available users" );
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while getting user at index " + index );
        }
    }

	User getUser(String strUsername, String strPassword) {
		Iterator iter = users.iterator();
		User currentUser = null;
		while( iter.hasNext() ){
			currentUser = (User)iter.next();
			if( currentUser.getUsername().equals(strUsername) &&
					currentUser.getPassword().equals(strPassword) ) return currentUser;
		}
		return null;
	}

    public boolean insertUser( User user ){
        if( user == null ) return false;
        return users.add( user );
    }

    public boolean insertUser( String username, String password, int type ) throws ZooException{
        return users.add( new User(username, password, type) );
    }

    public void insertUser( int dest, User user ) throws ZooException{
        try{
            users.add( dest, user );
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while inserting user: Index " + dest
                    + " is out of bounds. There are " + size() + " available users" );
        }
    }

    public void insertUser( int dest, String username, String password, int type ) throws ZooException{
        try{
            users.add( dest, new User(username, password, type) );
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while inserting user: Index " + dest
                    + " is out of bounds. There are " + size() + " available users" );
        }
    }

    public User removeUser( int index ) throws ZooException{
        try{
            return users.remove( index );
        }
        catch( IndexOutOfBoundsException ex ){
            throw new ZooException( ex, "Error while removing user: Index " + index
                    + " is out of bounds. There are " + size() + " available users" );
        }
    }

    public void clearRepository(){
        renewList();
    }


    public void openDataBase( String file ) throws ZooException{
        renewList();
        addDataBase( file );
    }

    public void addDataBase( String file ) throws ZooException{
        openFileForInput( file );

        try{
            while( true ){
                users.add( (User) input.readObject() );
            }
        }
        catch( EOFException ex ){
            // File finished.
            return;
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while reading from file " + file );
        }
        finally{
            closeFileForInput();
        }
    }

    public void saveDataBase( String file ) throws ZooException{
        try{
            openFileForOutput(file);

            Iterator iter = users.iterator();
            while( iter.hasNext() ){
                output.writeObject( iter.next() );
            }
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while saving repository to file " + file );
        }
        finally{
            closeFileForOutput();
        }
    }

    public String toString(){
        if( users.isEmpty() ) return new String( "" );

        StringBuffer buffer = new StringBuffer();

        Iterator iter = users.iterator();
        int number = 0;
        while( iter.hasNext() ){
            buffer.append( "\nUser No " + ++number + ":\n" );
            buffer.append( iter.next() );
        }

        return buffer.toString();
    }

    public int size(){
        return users.size();
    }


    //// Private methods
    //
    private void openFileForInput( String file ) throws ZooException{
        try{
            input = new ObjectInputStream( new FileInputStream( file ) );
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while opening file " + file + " for input." );
        }
    }

    private void openFileForOutput( String file ) throws ZooException{
        try{
            output = new ObjectOutputStream( new FileOutputStream( file ) );
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while opening file " + file + " for output" ) ;
        }
    }

    private void closeFileForInput() throws ZooException{
        try {
            if( input != null ){
                input.close();
            }
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while closing file for input." );
        }
    }

    private void closeFileForOutput() throws ZooException{
        try {
            if( output != null ){
                output.flush();
                output.close();
            }
        }
        catch( Exception ex ){
            throw new ZooException( ex, "Error while closing file for output." );
        }
    }

    private void renewList(){
        users = new LinkedList<User>();
    }

}
